// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#if !defined(IW_GAME_CAMERA_H_)
#define IW_GAME_CAMERA_H_

#include "s3e.h"
#include "IwGx.h"
#include "IwGeom.h"

#include "IwGameUtil.h"


//
//
//  CIwGameCamera - A camera is generally used to display a portion of a game scene, usually tracking some actor
//
//
class CIwGameCamera
{
public:
	// Properties
protected:
	unsigned int			NameHash;		// Hashed name of this camera
	CIwMat2D				Transform;		// The combined camera transform
	CIwFVec2				Position;		// Position of view within scene
	float					Scale;			// Cameras scale
	float					Angle;			// Cameras angle
	bool					TransformDirty;	// Marks camera transform needs rebuilding
public:
	void					setName(const char* name)			{ NameHash = IwHashString(name); }
	unsigned int			getNameHash()						{ return NameHash; }
	CIwMat2D&				getTransform()						{ return Transform; }
	void					setPosition(float x, float y)		{ Position.x = x; Position.y = y; TransformDirty = true; }
	CIwFVec2				getPosition() const					{ return Position; }
	void					setScale(float scale)				{ Scale = scale; TransformDirty = true; }
	float					getScale() const					{ return Scale; }
	void					setAngle(float angle)				{ Angle = angle; TransformDirty = true; }
	float					getAngle() const					{ return Angle; }
	void					forceTransformDirty()				{ TransformDirty = true; }
	bool					isTransformDirty() const			{ return TransformDirty; }
	// Properties end

private:

public:
	CIwGameCamera() : Position(0, 0), Scale(1.0f), Angle(0), TransformDirty(true) {}
	virtual ~CIwGameCamera()	{}

	// Updates the camera
	virtual void	Update();

	// Event handlers
};




#endif // IW_GAME_CAMERA_H_