// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#include "IwGameHttp.h"

//
//
// CIwGameHttpRequest - implementation
//
//
void CIwGameHttpRequest::EndRequest(int error)
{
	Error = error;
	Parent->setNoRequest();
	Processed = true;

	if (ContentAvailableCallback != NULL)
		ContentAvailableCallback((void*)this, ContentAvailableCallbackData);
}

//
//
// CIwGameHttpManager - implementation
//
//

CDECLARE_SINGLETON(CIwGameHttpManager)

void CIwGameHttpManager::Init()
{
	HttpObject = new CIwHTTP;

	CurrentRequest = NULL;
}

void CIwGameHttpManager::Release()
{
	Requests.clear();

	if (HttpObject != NULL)
	{
		HttpObject->Cancel();
		delete HttpObject;
		HttpObject = NULL;
	}

	CurrentRequest = NULL;
}

static int32 GotData(void*, void*)
{
    // This is the callback indicating that a ReadContent call has completed.  Either we've finished, or a bigger buffer is needed.  If the correct ammount of data was supplied initially,
	// then this will only be called once. However, it may well be called several times when using chunked encoding.
	CIwGameHttpManager* manager = IW_GAME_HTTP_MANAGER;
	if (manager == NULL)
		return 0;
	CIwGameHttpRequest* req = manager->getCurrentRequest();

    // Firstly see if there's an error condition.
    if (manager->GetStatus() == S3E_RESULT_ERROR)
    {
        // Something has gone wrong
		req->EndRequest(-3);
    }
	else
	if (manager->GetContentReceived() != manager->GetContentLength())
	{
		// We have some data but not all of it. We need more space.
		int len = req->getContentLength();
		int old_len = len;

		// If iwhttp has a guess how big the next bit of data is (this basically means chunked encoding is being used), allocate that much space. Otherwise guess.
		if (len < manager->GetContentExpected())
			len = manager->GetContentExpected();
		else
			len += 1024;

        // Allocate some more space and fetch the data.
		req->reallocContent(len);
		manager->ReadContent(req->getContent().str() + old_len, len - old_len, GotData);
	}
    else
    {
        // All data rerteived
		req->EndRequest(0);
    }
	return 0;
}

//
// Called when the response headers have been received
//
static int32 GotHeaders(void*, void*)
{
	CIwGameHttpManager* manager = IW_GAME_HTTP_MANAGER;
	if (manager == NULL)
		return 0;
	CIwGameHttpRequest* req = manager->getCurrentRequest();

    if (manager->GetStatus() == S3E_RESULT_ERROR)
    {
        // Something has gone wrong
		req->EndRequest(-2);
    }
    else
    {
		req->setError(0);

        // Depending on how the server is communicating the content length, we may actually know the length of the content, or we may know the length of the first part of it, or we may
        // know nothing. ContentExpected always returns the smallest possible size of the content, so allocate that much space for now if it's non-zero. If it is of zero size, the server
        // has given no indication, so we need to guess. We'll guess at 1k.
		int len = manager->GetContentExpected();
		if (len <= 0)
		{
			len = 1024;
		}

		req->allocContent(len);

		manager->ReadContent(req->getContent().str(), req->getContentLength(), GotData, NULL);
    }

	return 0;
}

bool CIwGameHttpManager::GetHeader(const char* header_name, CIwGameString* header_data)
{
	std::string type;
	if (!IW_GAME_HTTP_MANAGER->getHttpObject()->GetHeader(header_name, type))
		return false;

	header_data->setString(type.c_str());

	return true;
}

void CIwGameHttpManager::Update()
{
	typedef CIwList<CIwGameHttpRequest*>::iterator _iterator;

	if (CurrentRequest == NULL)
	{
		for(_iterator it = Requests.begin(); it != Requests.end(); it++)
		{
			CIwGameHttpRequest* req = *it;

			// If request has not been processed then process it
			if (!req->getProcessed())
			{
				CurrentRequest = req;

//				if (req->getBody().c_str() == NULL)
				if (!req->isPOST())
				{
					// Process the GET request
					if (HttpObject->Get(req->getURI().c_str(), GotHeaders, NULL) != S3E_RESULT_SUCCESS)
					{
						// Failed so set error and free up manager to process another request
						req->EndRequest(-1);
						break;
					}
				}
				else
				{
					// Process the POST request
					req->ApplyHeaders(HttpObject);
					if (HttpObject->Post(req->getURI().c_str(), req->getBody().c_str(), req->getBody().GetLength(), GotHeaders, NULL) != S3E_RESULT_SUCCESS)
					{
						// Failed so set error and free up manager to process another request
						req->EndRequest(-1);
						break;
					}
				}
				break;
			}
		}
	}
}

